@@ -2,7 +2,7 @@ source("data.R")
 
 #### interpolateLinear ####
 expect_identical(
-  DTSg$new(DT1)$colapply(interpolateLinear, cols = "col2")$values()[["col2"]],
+  DTSg$new(DT1)$colapply(interpolateLinear, cols = "col2")$values(TRUE)[["col2"]],
   seq(1, 15, by = 2),
   info = '"interpolateLinear" works correctly'
 )
